/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package nbmdownloader;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author toru
 */
public class Downloader {
    private List<URL> urls = new ArrayList<URL>();
    private URL xmlUrl;

    private static final Logger logger = Logger.getLogger(Downloader.class.getName());

    public Downloader(URL aXmlUrl, Iterable<String> anUrls) {
        xmlUrl = aXmlUrl;
        for (String urlText : anUrls) {
            try {
                URL url = new URL(urlText);
                urls.add(url);
            } catch (MalformedURLException e) {
                logger.info(urlText + " cannot be malformed, retry as relative URL");
                try {
                    URL url = new URL(aXmlUrl, urlText);
                    urls.add(url);
                } catch (MalformedURLException ex) {
                    logger.warning("retry failed, relative to " + aXmlUrl + ", " + urlText + " cannot be malformed");
                }
            }
        }
    }

    public void downloadAll() {
        download(xmlUrl);
        for (URL url : urls) {
            download(url);
        }
    }

    public void download(URL anUrl) {
        BufferedInputStream inStream = null;
        BufferedOutputStream outStream = null;
        try {
            inStream = new BufferedInputStream(anUrl.openStream());
            File file = new File(anUrl.getFile());
            outStream = new BufferedOutputStream(new FileOutputStream(file.getName()));
            byte[] buffer = new byte[1024];
            int sum = 0;
            while (true) {
                System.err.print('.');
                int ret = inStream.read(buffer);
                logger.finest("buffer read size:" + ret);
                if (ret <= 0) {
                    break;
                }
                outStream.write(buffer, 0, ret);
                sum += ret;
            }
            logger.fine("total read size:" + sum);
        } catch (IOException ex) {
            Logger.getLogger(Downloader.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                outStream.close();
            } catch (IOException ex) {
                Logger.getLogger(Downloader.class.getName()).log(Level.SEVERE, null, ex);
            }
            try {
                inStream.close();
            } catch (IOException ex) {
                Logger.getLogger(Downloader.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
}
