/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package nbmdownloader;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * NetBeansのモジュールダウンロード情報を定義するXMLファイルを解析し、
 * 必要なモジュールファイル一覧を取得する。
 *
 * @author toru
 */
public class ModuleParser {
    private URL xmlUrl;
    private XMLStreamReader reader;
    private static final Logger logger = Logger.getLogger(ModuleParser.class.getName());
    private List<String> moduleUrls = new ArrayList<String>();

    /**
     * 引数に指定されたURLからXMLファイルを取得する。
     * @param anUrl
     */
    public ModuleParser(URL anUrl) throws IOException {
        this(anUrl.openConnection().getInputStream());
        xmlUrl = anUrl;
    }

    ModuleParser(InputStream aStream) throws IOException {
        logger.fine("Constructed");
        XMLInputFactory factory = XMLInputFactory.newInstance();
        BufferedInputStream bufStream = new BufferedInputStream(aStream);
        try {
            reader = factory.createXMLStreamReader(bufStream);
        } catch (XMLStreamException ex) {
            logger.log(Level.SEVERE, null, ex);
            throw new IOException("XML Parse Error", ex);
        }
    }

    /**
     * XMLファイルをパーズし、ダウンロード対象ファイルURLを抜き出す。
     * StAX API（Cursor API）を用いて実装する。
     * module要素の属性distributionの文字列を抽出する。
     */
    public void parse() {
        logger.finer("Entering");
        try {
            boolean isModuleElement = false;
            for (; reader.hasNext(); reader.next()) {
                if (reader.getEventType() == XMLStreamConstants.START_ELEMENT &&
                        reader.getLocalName().equals("module")) {
                    logger.log(Level.FINE, "START_ELEMENT:{0} has {1} attributes",
                            new Object[] {reader.getLocalName(), reader.getAttributeCount()});
                    for (int i=0; i<reader.getAttributeCount(); i++) {
                        if (reader.getAttributeLocalName(i).equals("distribution")) {
                            logger.log(Level.INFO, "found 'distribution' attribute");
                            moduleUrls.add(reader.getAttributeValue(i));
                        }
                    }
                }
            }
        } catch (XMLStreamException ex) {
            logger.log(Level.SEVERE, null, ex);
        }
    }

    public List<String> getUrlList() {
        return Collections.unmodifiableList(moduleUrls);
    }

    public URL getXmlUrl() {
        return xmlUrl;
    }
}
